import asyncio
import os

from dotenv import load_dotenv
from langchain_openai import ChatOpenAI
from mcp_use import MCPAgent, MCPClient

ENTITY_USER_ID = os.getenv("ENTITY_USER_ID", "1e6a8edf-065d-4f9e-8525-6c6348cae208")
AUTH_SERVICE_URL = os.getenv("AUTH_SERVICE_URL", "http://0.0.0.0:8888/token")
MONITE_API_BASE = os.getenv("MONITE_AI_API_BASE", "https://api.dev.monite.com/v1/mcp")


async def main():
    load_dotenv()

    config = {
        "mcpServers": {
            "monite-mcp": {
                "command": "uv",
                "args": ["run", "monite-mcp"],
                "env": {
                    "ENTITY_USER_ID": ENTITY_USER_ID,
                    "AUTH_SERVICE_URL": AUTH_SERVICE_URL,
                    "MONITE_AI_API_BASE": MONITE_API_BASE,
                },
            }
        }
    }

    client = MCPClient.from_dict(config)

    llm = ChatOpenAI(model="gpt-4o")

    agent = MCPAgent(llm=llm, client=client, max_steps=30)

    while True:
        try:
            user_prompt = input("Enter your question (or 'exit' to quit): ")
            if user_prompt.lower() == "exit":
                break

            result = await agent.run(user_prompt)
            print(result)
            print("-" * 50)
        except KeyboardInterrupt:
            print("\nExiting...")
            break
        except Exception as e:
            print(f"Error: {e}")


if __name__ == "__main__":
    asyncio.run(main())



import asyncio
import os
from logging import getLogger

import dotenv
from pydantic_ai import Agent
from pydantic_ai.mcp import MCPServerStdio

logger = getLogger(__name__)

dotenv.load_dotenv()

ENTITY_USER_ID = os.getenv("ENTITY_USER_ID", "1e6a8edf-065d-4f9e-8525-6c6348cae208")
AUTH_SERVICE_URL = os.getenv("AUTH_SERVICE_URL", "http://0.0.0.0:8888/token")


async def main():
    monite_mcp_server = MCPServerStdio(
        "uv",
        args=[
            "run",
            "monite-mcp",
        ],
        env={
            "ENTITY_USER_ID": ENTITY_USER_ID,
            "AUTH_SERVICE_URL": AUTH_SERVICE_URL,
        },
    )

    agent = Agent(
        "openai:gpt-4o",
        mcp_servers=[monite_mcp_server],
    )

    async with agent.run_mcp_servers():
        while True:
            try:
                user_prompt = input("Enter your question (or 'exit' to quit): ")
                if user_prompt.lower() == "exit":
                    break

                result = await agent.run(user_prompt)
                print(result.output)
                print("-" * 50)
            except KeyboardInterrupt:
                print("\nExiting...")
                break
            except Exception as e:
                print(f"Error: {e}")


if __name__ == "__main__":
    asyncio.run(main())



import os

USER_AGENT = "monite-mcp/0.1"
MONITE_API_VERSION = "2024-05-25"

MONITE_AI_API_BASE = os.getenv(
    "MONITE_AI_API_BASE", "https://api.sandbox.monite.com/v1/mcp"
)

ENTITY_USER_ID = os.getenv("ENTITY_USER_ID")
if not ENTITY_USER_ID:
    raise SystemExit(
        "ENTITY_USER_ID environment variable is not set. "
        "It is required to run the Monite MCP server."
    )

AUTH_SERVICE_URL = os.getenv("AUTH_SERVICE_URL")
if not AUTH_SERVICE_URL:
    raise SystemExit(
        "AUTH_SERVICE_URL environment variable is not set. "
        "It is required to run the Monite MCP server."
    )



import logging
import os
from uuid import UUID

import dotenv
import httpx
import uvicorn
from fastapi import FastAPI, HTTPException, Query
from pydantic import BaseModel

# ##############################################################################
# IMPORTANT SECURITY DISCLAIMER:
# This script (`auth_service.py`) is provided as a simplified example
# for demonstration purposes only.
#
# In a real-world/production scenario, this authentication service MUST NOT
# be exposed publicly as is. Doing so could lead to critical security
# vulnerabilities. An attacker who can guess or obtain an `entity_user_id`
# could potentially gain unauthorized access by retrieving authentication tokens.
#
# A production-ready authentication service should:
# 1. Run in a private network or a securely restricted environment.
# 2. Implement robust authentication and authorization mechanisms beyond
#    just an `entity_user_id` to protect the token issuance endpoint.
# 3. Implement token caching mechanisms that respect token lifecycles
#    to reduce load on the Monite API and improve performance.
# 4. Have comprehensive logging, monitoring, and rate-limiting in place.
#
# Please ensure you understand these risks before deploying/using
# this exampple of authentication service.
# ##############################################################################

logging.basicConfig(
    level=logging.INFO, format="%(asctime)s - %(name)s - %(levelname)s - %(message)s"
)
logger = logging.getLogger(__name__)

dotenv.load_dotenv()

CLIENT_ID = os.getenv("CLIENT_ID")
CLIENT_SECRET = os.getenv("CLIENT_SECRET")
MONITE_API_BASE_URL = os.getenv("MONITE_API_BASE_URL", "https://api.dev.monite.com/v1")


if not CLIENT_ID:
    raise SystemExit("CLIENT_ID environment variable is not set.")
if not CLIENT_SECRET:
    raise SystemExit("CLIENT_SECRET environment variable is not set.")


app = FastAPI(
    title="Monite Auth Service",
    description="Provides Monite entity_user tokens using client credentials and entity_user_id.",
    version="0.1.0",
)


class MoniteAuthToken(BaseModel):
    access_token: str
    token_type: str
    expires_in: int


async def fetch_monite_entity_user_token(entity_user_id: UUID) -> MoniteAuthToken:
    """
    Fetches an entity user token from the Monite API using the
    CLIENT_ID and CLIENT_SECRET.
    """

    token_url = f"{MONITE_API_BASE_URL}/auth/token"
    payload = {
        "client_id": CLIENT_ID,
        "client_secret": CLIENT_SECRET,
        "entity_user_id": str(entity_user_id),
        "grant_type": "entity_user",
    }

    async with httpx.AsyncClient() as client:
        try:
            logger.info(
                f"Requesting entity_user token for {entity_user_id} from {token_url}"
            )
            response = await client.post(token_url, json=payload, timeout=10)
            response.raise_for_status()
        except httpx.RequestError as exc:
            logger.error(
                f"HTTP request failed when contacting Monite for {entity_user_id}: {exc}"
            )
            raise HTTPException(
                status_code=503,
                detail=f"Error connecting to Monite API: {exc}",
            )
        except httpx.HTTPStatusError as exc:
            logger.error(
                f"Monite API returned an error for {entity_user_id}: {exc.response.status_code} - {exc.response.text}"
            )
            detail = f"Monite API error: {exc.response.status_code}"
            try:
                error_details = exc.response.json()
                detail += f" - {error_details}"
            except Exception:
                detail += f" - {exc.response.text}"
            raise HTTPException(status_code=exc.response.status_code, detail=detail)

    response_json = response.json()
    logger.info(f"Successfully obtained token for entity_user_id: {entity_user_id}")
    return MoniteAuthToken(
        access_token=response_json["access_token"],
        token_type=response_json["token_type"],
        expires_in=response_json.get("expires_in"),
    )


@app.get(
    "/token",
    summary="Get Monite Entity User Token",
    response_model=MoniteAuthToken,
    description="Fetches an entity_user token from Monite for the specified entity_user_id. ",
)
async def get_token_endpoint(
    entity_user_id: UUID = Query(
        ...,
        description="The ID of the Entity User for whom the token is being generated.",
    ),
):
    try:
        token_data = await fetch_monite_entity_user_token(entity_user_id)
        return token_data
    except HTTPException as http_exc:
        raise http_exc
    except Exception:
        logger.exception(
            f"Unexpected error while processing token request for {entity_user_id}",
            exc_info=True,
        )
        raise HTTPException(status_code=500, detail="Internal server error.")


if __name__ == "__main__":
    logger.info(
        f"Starting Monite Auth Service for Monite Environment: {MONITE_API_BASE_URL}"
    )
    uvicorn.run(app, host="0.0.0.0", port=8888)



import json
from typing import Annotated, Any, Literal

import httpx
from mcp.server.fastmcp import FastMCP
from pydantic import Field

from monite_mcp.schemas import MoniteAuthToken
from monite_mcp.settings import (
    AUTH_SERVICE_URL,
    ENTITY_USER_ID,
    MONITE_AI_API_BASE,
    MONITE_API_VERSION,
    USER_AGENT,
)

mcp = FastMCP("monite-mcp")


McpError = str


async def fetch_auth_token() -> MoniteAuthToken:
    async with httpx.AsyncClient() as client:
        response = await client.get(
            AUTH_SERVICE_URL,  # type: ignore
            params={"entity_user_id": ENTITY_USER_ID},
        )
        response.raise_for_status()
        return MoniteAuthToken(
            access_token=response.json()["access_token"],
            token_type=response.json()["token_type"],
            expires_in=response.json()["expires_in"],
        )


async def make_get_request(
    url: str,
    monite_auth_token: MoniteAuthToken,
    params: dict[str, Any] = {},
) -> dict[str, Any] | McpError:
    headers = {
        "User-Agent": USER_AGENT,
        "Accept": "application/json",
        "x-monite-version": MONITE_API_VERSION,
        "Authorization": f"{monite_auth_token.token_type} {monite_auth_token.access_token}",
    }
    async with httpx.AsyncClient() as client:
        try:
            response = await client.get(
                url,
                headers=headers,
                timeout=30.0,
            )
            response.raise_for_status()
            return response.json()
        except Exception as e:
            return f"Failed to make request to monite-ai: {e}"


@mcp.tool(
    name="get_counterparties_list",
    description="Get a list of counterparties, optionally filtered by vendor or customer status.",
)
async def get_counterparties_list(
    is_vendor: bool = False,
    is_customer: bool = False,
) -> str:
    url = f"{MONITE_AI_API_BASE}/counterparties_list"
    monite_auth_token = await fetch_auth_token()
    data = await make_get_request(
        url,
        monite_auth_token,
        params={
            "is_vendor": is_vendor,
            "is_customer": is_customer,
        },
    )
    return json.dumps(data)


@mcp.tool(
    name="get_payables_waiting_to_be_paid",
    description="Retrieves all payables with status 'waiting_to_be_paid' (payables that need to be paid by us)",
)
async def get_payables_waiting_to_be_paid() -> str:
    url = f"{MONITE_AI_API_BASE}/payables_waiting_to_be_paid"
    monite_auth_token = await fetch_auth_token()
    data = await make_get_request(
        url,
        monite_auth_token,
    )
    return json.dumps(data)


@mcp.tool(
    name="get_pending_receivables_total_amount",
    description="Retrieval of the total amount of pending receivables for a given period.",
)
async def get_pending_receivables_total_amount(
    period: Literal["THIS_MONTH", "LAST_MONTH", "NEXT_MONTH"] = "THIS_MONTH",
) -> str:
    """Retrieves the total amount of pending receivables for a given period.

    Args:
        period: The period to retrieve the total amount of pending receivables for. Available options: THIS_MONTH, LAST_MONTH, NEXT_MONTH.
    """
    url = f"{MONITE_AI_API_BASE}/pending_receivables_total_amount"
    monite_auth_token = await fetch_auth_token()
    data = await make_get_request(
        url,
        monite_auth_token,
        params={"period": period},
    )
    return json.dumps(data)


@mcp.tool(
    name="get_overdue_receivables_total_amount",
    description="Retrieval of the total amount of overdue receivables.",
)
async def get_overdue_receivables_total_amount() -> str:
    url = f"{MONITE_AI_API_BASE}/overdue_receivables_total_amount"
    monite_auth_token = await fetch_auth_token()
    data = await make_get_request(url, monite_auth_token, params={})
    return json.dumps(data)


@mcp.tool(
    name="get_expired_receivables_count",
    description="Retrieval of the count of expired receivables for a given period.",
)
async def get_expired_receivables_count(
    period: Literal[
        "THIS_MONTH", "LAST_MONTH", "TWO_MONTHS_AGO", "ALL_TIME"
    ] = "LAST_MONTH",
) -> str:
    """Retrieves the count of expired receivables for a given period.

    Args:
        period: The period to retrieve the count of expired receivables for.
                Available options: THIS_MONTH, LAST_MONTH, TWO_MONTHS_AGO, ALL_TIME.
    """
    url = f"{MONITE_AI_API_BASE}/expired_receivables_count"
    monite_auth_token = await fetch_auth_token()
    data = await make_get_request(
        url,
        monite_auth_token,
        params={"period": period},
    )
    return json.dumps(data)


@mcp.tool(
    name="get_overdue_receivables",
    description="Retrieval of overdue receivables.",
)
async def get_overdue_receivables() -> str:
    url = f"{MONITE_AI_API_BASE}/overdue_receivables"
    monite_auth_token = await fetch_auth_token()
    data = await make_get_request(url, monite_auth_token, params={})
    return json.dumps(data)


@mcp.tool(
    name="get_owed_receivables",
    description="Retrieval of owed receivables based on status filters.",
)
async def get_owed_receivables(
    include_issued: Annotated[
        bool, Field(description="Whether to include issued receivables")
    ] = True,
    include_partially_paid: Annotated[
        bool, Field(description="Whether to include partially paid receivables")
    ] = True,
    include_overdue: Annotated[
        bool, Field(description="Whether to include overdue receivables")
    ] = True,
) -> str:
    url = f"{MONITE_AI_API_BASE}/owed_receivables"
    monite_auth_token = await fetch_auth_token()
    params = {
        "include_issued": include_issued,
        "include_partially_paid": include_partially_paid,
        "include_overdue": include_overdue,
    }
    data = await make_get_request(url, monite_auth_token, params=params)
    return json.dumps(data)


def main():
    mcp.run(transport="stdio")


if __name__ == "__main__":
    main()
